Package: GeoTox
Title: Spatiotemporal Mixture Risk Assessment
Version: 1.0.0
Authors@R: c(
    person("Skylar", "Marvel", , "skylar.marvel@nih.gov", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-2971-9743")),
    person("David", "Reif", , "david.reif@nih.gov", role = "aut",
           comment = c(ORCID = "0000-0001-7815-6767")),
    person("Kyle", "Messier", , "kyle.messier@nih.gov", role = "aut",
           comment = c(ORCID = "0000-0001-9508-9623")),
    person("Spatiotemporal Exposures and Toxicology Group", role = "cph")
  )
Description: Connecting spatiotemporal exposure to individual and
    population-level risk via source-to-outcome continuum modeling. The
    package, methods, and case-studies are described in Messier, Reif, and
    Marvel (2025) <doi:10.1186/s40246-024-00711-8> and Eccles et al.
    (2023) <doi:10.1016/j.scitotenv.2022.158905>.
License: MIT + file LICENSE
URL: https://niehs.github.io/GeoTox/, https://github.com/NIEHS/GeoTox
BugReports: https://github.com/NIEHS/GeoTox/issues
Depends: R (>= 4.4.0)
Imports: DBI, dbplyr, dplyr, duckdb, duckplyr, purrr, rlang, tibble,
        tidyr, tidyselect, truncnorm
Suggests: blob, ggplot2, ggridges, httk, httr2, knitr, quarto, readr,
        readxl, rmarkdown, sf, stringr, testthat (>= 3.0.0), withr, zip
VignetteBuilder: knitr, quarto
Config/Needs/website: rmarkdown, quarto
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-19 17:30:01 UTC; marvelsw
Author: Skylar Marvel [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2971-9743>),
  David Reif [aut] (ORCID: <https://orcid.org/0000-0001-7815-6767>),
  Kyle Messier [aut] (ORCID: <https://orcid.org/0000-0001-9508-9623>),
  Spatiotemporal Exposures and Toxicology Group [cph]
Maintainer: Skylar Marvel <skylar.marvel@nih.gov>
Repository: CRAN
Date/Publication: 2026-05-19 21:20:18 UTC
Built: R 4.6.0; ; 2026-05-19 22:23:15 UTC; unix
