#' @keywords internal
"_PACKAGE"

# Dummy function to avoid R CMD check notes about unused imports from dbplyr.
ignore_unused_imports <- function() {
  dbplyr::tbl_lazy
}

## usethis namespace: start
#' @importFrom rlang .data
#' @importFrom rlang .env
## usethis namespace: end
NULL
