# calc sensitivity

    Code
      sens_tbl
    Output
      # A tibble: 3 x 6
        assay_id sample_id GCA.Eff IA.Eff GCA.HQ.10 IA.HQ.10
           <dbl>     <dbl>   <dbl>  <dbl>     <dbl>    <dbl>
      1       NA         1    1.94   1.94      9.27     9.27
      2       NA         2    2.06   2.06     10.4     10.4 
      3       NA         3    2.13   2.13     11.3     11.3 

# other inputs

    Code
      sens_tbl
    Output
      # A tibble: 3 x 6
        assay_id sample_id GCA.Eff IA.Eff GCA.HQ.10 IA.HQ.10
           <dbl>     <dbl>   <dbl>  <dbl>     <dbl>    <dbl>
      1       NA         1    3.78   3.78    768.     768.  
      2       NA         2    1.97   1.97      9.55     9.55
      3       NA         3    3.82   3.82   4967     4967   

