# overwrite

    Code
      risk_tbl
    Output
      # A tibble: 1 x 6
        assay_id sample_id GCA.Eff IA.Eff GCA.HQ.10 IA.HQ.10
           <dbl>     <dbl>   <dbl>  <dbl>     <dbl>    <dbl>
      1       NA         1   0.452  0.452    712700   712700

