# normal risk

    Code
      risk_tbl
    Output
      # A tibble: 2 x 6
        assay_id sample_id GCA.Eff IA.Eff GCA.HQ.10  IA.HQ.10
           <dbl>     <dbl>   <dbl>  <dbl>     <dbl>     <dbl>
      1       NA         1   0.497  0.497  15220000  15220000
      2       NA         2   0.601  0.602 135500000 135500000

# risk sensitivity

    Code
      risk_tbl
    Output
      # A tibble: 2 x 6
        assay_id sample_id GCA.Eff IA.Eff GCA.HQ.10 IA.HQ.10
           <dbl>     <dbl>   <dbl>  <dbl>     <dbl>    <dbl>
      1       NA         1   0.590  0.591  15080000 15080000
      2       NA         2   0.596  0.597  89640000 89640000

