% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{geo_tox_data}
\alias{geo_tox_data}
\title{GeoTox Data}
\format{
A list with items:
\describe{
\item{exposure}{2020 AirToxScreen exposure concentrations for a subset of
chemicals in North Carolina counties.}
\item{dose_response}{Subset of chemicals curated by ICE cHTS as active
within a set of assays.}
\item{age}{County population estimates for 7/1/2020 in North Carolina.}
\item{obesity}{CDC PLACES obesity data for North Carolina counties in
2020.}
\item{simulated_css}{Simulated steady-state plasma concentrations for
various age groups and obesity status combinations.}
\item{boundaries}{County and state boundaries for North Carolina in 2020.}
}
}
\usage{
geo_tox_data
}
\description{
Sample data for use in vignettes and function examples. See the Package
Data vignette, \code{vignette("package_data", package = "GeoTox")}, for
details on how this data was gathered.
}
\keyword{datasets}
